@@ -1,23 +1,49 @@
-from typing import Union, Dict, Any, Optional, List, Tuple, Callable, Type, Mapping
-from typing import Iterable, Sequence, cast
-from types import GeneratorType
-from dataclasses import dataclass
-from configparser import ConfigParser, ExtendedInterpolation, MAX_INTERPOLATION_DEPTH
-from configparser import InterpolationMissingOptionError, InterpolationSyntaxError
-from configparser import NoSectionError, NoOptionError, InterpolationDepthError
-from configparser import ParsingError
-from pathlib import Path
-from pydantic import BaseModel, create_model, ValidationError, Extra
-from pydantic.main import ModelMetaclass
-from pydantic.fields import ModelField
-import srsly
-import catalogue
+import copy
 import inspect
 import io
-import copy
 import re
+import warnings
+from configparser import (
+    MAX_INTERPOLATION_DEPTH,
+    ConfigParser,
+    ExtendedInterpolation,
+    InterpolationDepthError,
+    InterpolationMissingOptionError,
+    InterpolationSyntaxError,
+    NoOptionError,
+    NoSectionError,
+    ParsingError,
+)
+from dataclasses import dataclass
+from pathlib import Path
+from types import GeneratorType
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
 
-from .util import Decorator
+import srsly
+
+try:
+    from pydantic.v1 import BaseModel, Extra, ValidationError, create_model
+    from pydantic.v1.fields import ModelField
+    from pydantic.v1.main import ModelMetaclass
+except ImportError:
+    from pydantic import BaseModel, create_model, ValidationError, Extra  # type: ignore
+    from pydantic.main import ModelMetaclass  # type: ignore
+    from pydantic.fields import ModelField  # type: ignore
+
+from .util import SimpleFrozenDict, SimpleFrozenList  # noqa: F401
 
 # Field used for positional arguments, e.g. [section.*.xyz]. The alias is
 # required for the schema (shouldn't clash with user-defined arg names)
@@ -43,6 +69,12 @@ class CustomInterpolation(ExtendedInterpolation):
             json_value = srsly.json_loads(value)
             if isinstance(json_value, str) and json_value not in JSON_EXCEPTIONS:
                 value = json_value
+        except ValueError:
+            if value and value[0] == value[-1] == "'":
+                warnings.warn(
+                    f"The value [{value}] seems to be single-quoted, but values "
+                    "use JSON formatting, which requires double quotes."
+                )
         except Exception:
             pass
         return super().before_read(parser, section, option, value)
@@ -118,7 +150,7 @@ class CustomInterpolation(ExtendedInterpolation):
                 else:
                     accum.append(v)
             else:
-                err = "'$' must be followed by '$' or '{', " "found: %r" % (rest,)
+                err = "'$' must be followed by '$' or '{', found: %r" % (rest,)
                 raise InterpolationSyntaxError(option, section, err)
 
     def _get_section_name(self, name: str) -> str:
@@ -213,7 +245,9 @@ class Config(dict):
                 if part == "*":
                     node = node.setdefault(part, {})
                 elif part not in node:
-                    err_title = f"Error parsing config section. Perhaps a section name is wrong?"
+                    err_title = (
+                        "Error parsing config section. Perhaps a section name is wrong?"
+                    )
                     err = [{"loc": parts, "msg": f"Section '{part}' is not defined"}]
                     raise ConfigValidationError(
                         config=self, errors=err, title=err_title
@@ -266,8 +300,6 @@ class Config(dict):
         # NOTE: This currently can't handle uninterpolated values like [${x.y}]!
         if isinstance(result, str) and VARIABLE_RE.search(value):
             result = value
-        if isinstance(result, list):
-            return [self._interpret_value(v) for v in result]
         return result
 
     def _get_section_ref(self, value: Any, *, parent: List[str] = []) -> Any:
@@ -347,7 +379,7 @@ class Config(dict):
         """Set overrides in the ConfigParser before config is interpreted."""
         err_title = "Error parsing config overrides"
         for key, value in overrides.items():
-            err_msg = "not a section value that can be overwritten"
+            err_msg = "not a section value that can be overridden"
             err = [{"loc": key.split("."), "msg": err_msg}]
             if "." not in key:
                 raise ConfigValidationError(errors=err, title=err_title)
@@ -407,11 +439,7 @@ class Config(dict):
                 if hasattr(value, "items"):
                     # Reference to a function with no arguments, serialize
                     # inline as a dict and don't create new section
-                    if (
-                        registry.is_promise(value)
-                        and len(value) == 1
-                        and is_kwarg
-                    ):
+                    if registry.is_promise(value) and len(value) == 1 and is_kwarg:
                         flattened.set(section_name, key, try_dump_json(value, node))
                     else:
                         queue.append((path + (key,), value))
@@ -492,7 +520,10 @@ def try_dump_json(value: Any, data: Union[Dict[str, dict], Config, str] = "") ->
         # Work around values that are strings but numbers
         value = f'"{value}"'
     try:
-        return srsly.json_dumps(value)
+        value = srsly.json_dumps(value)
+        value = re.sub(r"\$([^{])", "$$\1", value)
+        value = re.sub(r"\$$", "$$", value)
+        return value
     except Exception as e:
         err_msg = (
             f"Couldn't serialize config value of type {type(value)}: {e}. Make "
@@ -673,6 +704,7 @@ def copy_model_field(field: ModelField, type_: Any) -> ModelField:
         default=field.default,
         default_factory=field.default_factory,
         required=field.required,
+        alias=field.alias,
     )
 
 
@@ -881,6 +913,15 @@ class registry:
                     # created via config blocks), only use its values
                     validation[v_key] = list(validation[v_key].values())
                     final[key] = list(final[key].values())
+
+                    if ARGS_FIELD_ALIAS in schema.__fields__ and not resolve:
+                        # If we're not resolving the config, make sure that the field
+                        # expecting the promise is typed Any so it doesn't fail
+                        # validation if it doesn't receive the function return value
+                        field = schema.__fields__[ARGS_FIELD_ALIAS]
+                        schema.__fields__[ARGS_FIELD_ALIAS] = copy_model_field(
+                            field, Any
+                        )
             else:
                 filled[key] = value
                 # Prevent pydantic from consuming generator if part of a union
@@ -905,7 +946,12 @@ class registry:
             # manually because .construct doesn't parse anything
             if schema.Config.extra in (Extra.forbid, Extra.ignore):
                 fields = schema.__fields__.keys()
-                exclude = [k for k in result.__fields_set__ if k not in fields]
+                # If we have a reserved field, we need to use its alias
+                field_set = [
+                    k if k != ARGS_FIELD else ARGS_FIELD_ALIAS
+                    for k in result.__fields_set__
+                ]
+                exclude = [k for k in field_set if k not in fields]
         exclude_validation = set([ARGS_FIELD_ALIAS, *RESERVED_FIELDS.keys()])
         validation.update(result.dict(exclude=exclude_validation))
         filled, final = cls._update_from_parsed(validation, filled, final)
@@ -1040,4 +1086,10 @@ class registry:
         return create_model("ArgModel", **sig_args)
 
 
-__all__ = ["Config", "registry", "ConfigValidationError"]
+__all__ = [
+    "Config",
+    "registry",
+    "ConfigValidationError",
+    "SimpleFrozenDict",
+    "SimpleFrozenList",
+]
