@@ -1,19 +1,22 @@
 import inspect
+import pickle
 import platform
+from types import GeneratorType
+from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
 import catalogue
 import pytest
-from typing import Dict, Optional, Iterable, Callable, Any, Union, List, Tuple
-from types import GeneratorType
-import pickle
 
-from pydantic import BaseModel, StrictFloat, PositiveInt, constr
-from pydantic.types import StrictBool
+try:
+    from pydantic.v1 import BaseModel, PositiveInt, StrictFloat, constr
+    from pydantic.v1.types import StrictBool
+except ImportError:
+    from pydantic import BaseModel, StrictFloat, PositiveInt, constr  # type: ignore
+    from pydantic.types import StrictBool  # type: ignore
 
-from confection import ConfigValidationError, Config
+from confection import Config, ConfigValidationError
+from confection.tests.util import Cat, make_tempdir, my_registry
 from confection.util import Generator, partial
-from confection.tests.util import Cat, my_registry, make_tempdir
-
 
 EXAMPLE_CONFIG = """
 [optimizer]
@@ -287,6 +290,29 @@ bar = 1
     )
 
 
+def test_config_to_str_escapes():
+    section_str = """
+        [section]
+        node1 = "^a$$"
+        node2 = "$$b$$c"
+        """
+    section_dict = {"section": {"node1": "^a$", "node2": "$b$c"}}
+
+    # parse from escaped string
+    cfg = Config().from_str(section_str)
+    assert cfg == section_dict
+
+    # parse from non-escaped dict
+    cfg = Config(section_dict)
+    assert cfg == section_dict
+
+    # roundtrip through str
+    cfg_str = cfg.to_str()
+    assert "^a$$" in cfg_str
+    new_cfg = Config().from_str(cfg_str)
+    assert cfg == section_dict
+
+
 def test_config_roundtrip_bytes():
     cfg = Config().from_str(OPTIMIZER_CFG)
     cfg_bytes = cfg.to_bytes()
@@ -328,7 +354,7 @@ def test_validation_custom_types():
     def complex_args(
         rate: StrictFloat,
         steps: PositiveInt = 10,  # type: ignore
-        log_level: constr(regex="(DEBUG|INFO|WARNING|ERROR)") = "ERROR",
+        log_level: constr(regex="(DEBUG|INFO|WARNING|ERROR)") = "ERROR",  # noqa: F821
     ):
         return None
 
@@ -398,6 +424,28 @@ def test_make_config_positional_args():
     assert my_registry.resolve(cfg)["config"] == "^_^"
 
 
+def test_fill_config_positional_args_w_promise():
+    @my_registry.cats("catsie.v568")
+    def catsie_568(*args: str, foo: str = "bar"):
+        assert args[0] == "^(*.*)^"
+        assert foo == "baz"
+        return args[0]
+
+    @my_registry.cats("cat_promise.v568")
+    def cat_promise() -> str:
+        return "^(*.*)^"
+
+    cfg = {
+        "config": {
+            "@cats": "catsie.v568",
+            "*": {"promise": {"@cats": "cat_promise.v568"}},
+        }
+    }
+    filled = my_registry.fill(cfg, validate=True)
+    assert filled["config"]["foo"] == "bar"
+    assert filled["config"]["*"] == {"promise": {"@cats": "cat_promise.v568"}}
+
+
 def test_make_config_positional_args_complex():
     @my_registry.cats("catsie.v890")
     def catsie_890(*args: Optional[Union[StrictBool, PositiveInt]]):
@@ -770,7 +818,6 @@ def test_fill_config_dict_return_type():
 
 
 def test_deepcopy_config():
-    numpy = pytest.importorskip("numpy")
     config = Config({"a": 1, "b": {"c": 2, "d": 3}})
     copied = config.copy()
     # Same values but not same object
@@ -1287,6 +1334,7 @@ def test_config_fill_without_resolve():
     assert filled2["catsie"]["cute"] is True
     resolved = my_registry.resolve(filled2)
     assert resolved["catsie"] == "meow"
+
     # With unavailable function
     class BaseSchema2(BaseModel):
         catsie: Any
@@ -1379,3 +1427,29 @@ def test_config_overrides(greeting, value, expected):
     assert "${vars.a}" in str_cfg
     cfg = Config().from_str(str_cfg, overrides=overrides)
     assert expected in str(cfg)
+
+
+def test_warn_single_quotes():
+    str_cfg = """
+    [project]
+    commands = 'do stuff'
+    """
+
+    with pytest.warns(UserWarning, match="single-quoted"):
+        Config().from_str(str_cfg)
+
+    # should not warn if single quotes are in the middle
+    str_cfg = """
+    [project]
+    commands = some'thing
+    """
+    Config().from_str(str_cfg)
+
+
+def test_parse_strings_interpretable_as_ints():
+    """Test whether strings interpretable as integers are parsed correctly (i. e. as strings)."""
+    cfg = Config().from_str(
+        f"""[a]\nfoo = [${{b.bar}}, "00${{b.bar}}", "y"]\n\n[b]\nbar = 3"""  # noqa: F541
+    )
+    assert cfg["a"]["foo"] == [3, "003", "y"]
+    assert cfg["b"]["bar"] == 3
